;;; Copyright © 2020 Collin J. Doering <collin@rekahsoft.ca>
;;;
;;; This file is part of the GNU Guix channel rekahsoft-guix
;;;
;;; The rekahsoft-guix channel for GNU Guix is free software; you can
;;; redistribute it and/or modify it under the terms of the GNU General Public
;;; License as published by the Free Software Foundation; either version 3 of
;;; the License, or (at your option) any later version.
;;;
;;; The rekahsoft-guix channel for GNU Guix is distributed in the hope that it
;;; will be useful, but WITHOUT ANY WARRANTY; without even the implied
;;; warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License along
;;; with the rekahsoft-guix channel for GNU Guix.  If not, see
;;; <http://www.gnu.org/licenses/>.

(define-module (rekahsoft-gnu packages emacs-xyz)
  #:use-module ((guix licenses)
                #:prefix license:)
  #:use-module (guix packages)
  #:use-module (guix download)
  #:use-module (guix git-download)
  #:use-module (guix build-system copy)
  #:use-module (guix build-system gnu)
  #:use-module (guix build-system emacs)
  #:use-module (guix utils)
  #:use-module (gnu packages)
  #:use-module (gnu packages emacs)
  #:use-module (gnu packages emacs-xyz)
  #:use-module (gnu packages curl)
  #:use-module (gnu packages mail)
  #:use-module (gnu packages statistics)
  #:use-module (gnu packages erlang)
  #:use-module (gnu packages ocaml)
  #:use-module (srfi srfi-1)
  #:use-module (ice-9 match))

;;;
;;; Emacs hacking.
;;;

(define-public emacs-helm-tramp
  (package
    (name "emacs-helm-tramp")
    (version "1.3.9")
    (source (origin
              (method url-fetch)
              (uri (string-append
                    "https://stable.melpa.org/packages/helm-tramp-" version
                    ".el"))
              (sha256
               (base32
                "1nhlfq113ngcb74n76z8c2wnki14vls18mml36iwi8y9k63flkcd"))))
    (build-system emacs-build-system)
    (propagated-inputs `(("emacs-helm" ,emacs-helm)))
    (home-page "https://github.com/masasam/emacs-helm-tramp")
    (synopsis "Tramp helm interface for ssh, docker, vagrant")
    (description
     "Provides interfaces of Tramp.
You can also use tramp with helm interface as root
If you use it with docker-tramp, you can also use docker with helm interface
If you use it with vagrant-tramp, you can also use vagrant with helm interface
")
    (license license:gpl3)))

(define-public emacs-rebox2
  (package
    (name "emacs-rebox2")
    (version "20121113.1300")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://melpa.org/packages/rebox2-" version
                                  ".el"))
              (sha256
               (base32
                "0ji5x2hnjklz9y4jd11hcfncbcfzgq1dg8r3s7dkrf696y4ziwwy"))))
    (build-system emacs-build-system)
    (home-page "https://github.com/lewang/rebox2")
    (synopsis "Inserting and modification of comment boxes in various styles")
    (description
     "Rebox2 provides major and minor modes for inserting and modification of
comment box is various text styles.")
    (license #f)))

;; TODO: ./rekahsoft-gnu/packages/emacs-xyz.scm:172:6: emacs-ibuffer-vc@20181225.2227: URI https://melpa.org/packages/ibuffer-vc-20181225.2227.el not reachable: 404 ("Not Found")
(define-public emacs-ibuffer-vc
  (package
    (name "emacs-ibuffer-vc")
    (version "20181225.2227")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://melpa.org/packages/ibuffer-vc-"
                                  version ".el"))
              (sha256
               (base32
                "16nry1631g4draqh09r2rq92xpv5vsyxx9yd7r5i2m6kblqh4y3f"))))
    (build-system emacs-build-system)
    (home-page "https://github.com/purcell/ibuffer-vc")
    (synopsis "Group ibuffer's list by VC project, or show VC status")
    (description
     "Adds functionality to ibuffer for grouping buffers by their parent
vc root directory, and for displaying and/or sorting by the vc
status of listed files.
")
    (license license:gpl3)))

;; TODO: ./rekahsoft-gnu/packages/emacs-xyz.scm:198:6: emacs-highlight-indent-guides@20190108.3: URI https://melpa.org/packages/highlight-indent-guides-20190108.3.el not reachable: 404 ("Not Found")
(define-public emacs-highlight-indent-guides
  (package
    (name "emacs-highlight-indent-guides")
    (version "20190108.3")
    (source (origin
              (method url-fetch)
              (uri (string-append
                    "https://melpa.org/packages/highlight-indent-guides-"
                    version ".el"))
              (sha256
               (base32
                "02imngb4y6f9vp4jxiwdbdcjxllvjwaa2rlh61zy49n2wivrlwh0"))))
    (build-system emacs-build-system)
    (home-page "https://github.com/DarthFennec/highlight-indent-guides")
    (synopsis "Minor mode to highlight indentation")
    (description
     "This minor mode highlights indentation levels via font-lock.  Indent widths
are dynamically discovered, which means this correctly highlights in any
mode, regardless of indent width, even in languages with non-uniform
indentation such as Haskell.  This mode works properly around hard tabs and
mixed indentation, and it behaves well in large buffers.")
    (license #f)))

(define-public emacs-intel-hex-mode
  (package
    (name "emacs-intel-hex-mode")
    (version "20180423.31")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://melpa.org/packages/intel-hex-mode-"
                    version ".el"))
              (sha256
               (base32
                "1nvwjk686wna96srk3y9xniixas729x6m1rcjnc8lz1zkap9z9bv"))))
    (build-system emacs-build-system)
    (home-page "https://github.com/mschuldt/intel-hex-mode")
    (synopsis "Mode for Intel Hex files")
    (description
     "Use this mode for editing files in the intel hex format.  The
intel-hex-mode will do font locking, and calculate checksums.")
    (license license:gpl2)))

(define-public emacs-highlight-parentheses
  (package
    (name "emacs-highlight-parentheses")
    (version "20180704.1102")
    (source (origin
              (method url-fetch)
              (uri (string-append
                    "https://melpa.org/packages/highlight-parentheses-"
                    version ".el"))
              (sha256
               (base32
                "0dxnw7d966pm44a54rmd40vrrps56jaayrc8wwj7wxl2imdzqzbl"))))
    (build-system emacs-build-system)
    (home-page "https://github.com/tsdh/highlight-parentheses.el")
    (synopsis "Highlight surrounding parentheses")
    (description
     "Enable the mode using M-x highlight-parentheses-mode or by adding it to
a hook.")
    (license license:gpl2)))

(define-public emacs-quack
  (package
    (name "emacs-quack")
    (version "20181106.1301")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://melpa.org/packages/quack-" version
                                  ".el"))
              (sha256
               (base32
                "0ph6zh91kasgbvi425a5m1hz94pxk6qa3srpy4khifsbn7cwiyga"))))
    (build-system emacs-build-system)
    (home-page "https://www.neilvandyke.org/quack/")
    (synopsis "Enhanced support for editing and running Scheme code")
    (description
     "Quack enhances Emacs support for Scheme programming.  The name
Quack was a play on DrScheme.")
    (license license:gpl2)))

(define-public emacs-hamlet-mode
  (package
    (name "emacs-hamlet-mode")
    (version "20131208.724")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://melpa.org/packages/hamlet-mode-"
                                  version ".el"))
              (sha256
               (base32
                "1r6nxi3d6bmnxrjjsam79vsprfwlnbhpag98vb1hj05yqp13l55v"))))
    (build-system emacs-build-system)
    (propagated-inputs `(("emacs-dash" ,emacs-dash)
                         ("emacs-s" ,emacs-s)))
    (home-page "https://github.com/lightquake/hamlet-mode")
    (synopsis "Hamlet editing mode")
    (description
     "An Emacs major mode for editing files written in Hamlet, a
Haskell compile-time HTML templating engine.  Currently it only provides
syntax highlighting.

Functions and variables with // in their name are private and may change or
go away at any time.")
    (license #f)))

(define-public emacs-helm-unicode
  (package
    (name "emacs-helm-unicode")
    (version "20180608.1407")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://melpa.org/packages/helm-unicode-"
                                  version ".el"))
              (sha256
               (base32
                "1q4jkak147srs2vpnn9hl82irm9bdvj3wwxygf09rqv2fsa8cl9w"))))
    (build-system emacs-build-system)
    (propagated-inputs `(("emacs-helm" ,emacs-helm)))
    (home-page "https://github.com/bomgar/helm-unicode")
    (synopsis "Helm command for unicode characters.")
    (description "A helm command for looking up unicode characters by name.")
    (license #f)))

(define-public emacs-vimish-fold
  (package
    (name "emacs-vimish-fold")
    (version "20200524.1729")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://melpa.org/packages/vimish-fold-"
                                  version ".el"))
              (sha256
               (base32
                "08p826zn4ris9b46r1d5z9v9jdj4lyr6kdmw4yhmnm9sk41z4bm4"))))
    (build-system emacs-build-system)
    (propagated-inputs `(("emacs-f" ,emacs-f)))
    (home-page "https://github.com/matsievskiysv/vimish-fold")
    (synopsis "Fold text like in Vim")
    (description
     "This is a package to perform text folding like in Vim.  It has the
following features:

* folding of active regions;
* good visual feedback: it's obvious which part of text is folded;
* persistence by default: when you kill a buffer your folds don't
  disappear;
* persistence scales well, you can work on hundreds of files with lots of
  folds without adverse effects;
* it does not break indentation;
* folds can be toggled from folded state to unfolded and back very
  easily;
* quick navigation between existing folds;
* you can use mouse to unfold folds (good for beginners and not only for
  them);
* for fans of code@{avy package}: you can use code@{avy} to fold text with minimal
  number of key strokes!")
    (license #f)))

(define-public emacs-grapnel
  (package
    (name "emacs-grapnel")
    (version "20131001.1534")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://melpa.org/packages/grapnel-"
                                  version ".el"))
              (sha256
               (base32
                "1mkkv6ldqwgw54da4b1j9rqy275fkl7lqvlq6h6ji3jvf9dfgvrc"))))
    (build-system emacs-build-system)
    (home-page "https://www.github.com/leathekd/grapnel")
    (synopsis "HTTP request lib with flexible callback dispatch")
    (description
     "Grapnel is an HTTP request library that uses a curl subprocess and
offers flexible callback dispatch.  Not only can you pass in an
alist of request outcomes to callback functions (see below) but you
can also override the dispatch function itself if the default one
doesn't suit your needs.  Further, grapnel will build the query
string, request data (i.e., POST body), and headers from alists
that are passed in.

An example:
(grapnel-retrieve-url
 \"www.google.com\"
 '((success . (lambda (res hdrs) (message \"%s\" res)))
   (failure . (lambda (res hdrs) (message \"Fail: %s\" res)))
   (error   . (lambda (res err)  (message \"Err: %s\" err))))
 \"GET\"
 '((q . \"ASIN B001EN71CW\")))")
    (license license:gpl3)))

(define-public emacs-ix
  (package
    (name "emacs-ix")
    (version "20131027.1629")
    (source (origin
              (method url-fetch)
              (uri (string-append "https://melpa.org/packages/ix-" version
                                  ".el"))
              (sha256
               (base32
                "165nr8cz2y0mpcav0bkc8kak5zji4fayrs9v9wdzn0nksq6wdbls"))))
    (build-system emacs-build-system)
    (propagated-inputs `(("emacs-grapnel" ,emacs-grapnel)
                         ("curl" ,curl)))
    (home-page "https://www.github.com/theanalyst/ix.el")
    (synopsis "Emacs client for http://ix.io pastebin")
    (description
     "ix.el is a simple emacs client to http://ix.io cmdline pastebin. At
the moment using the code@{ix} command on a selection sends the
selection to ix.io, entire buffer is sent if selection is inactive,
on success the url is notified in the minibuffer as well as saved
in the kill ring.

It is recommended to set a user name and token so that you can
later delete or replace a paste.  Set this via the variables
code@{ix-user} and code@{ix-token} via M-x customize-group RET ix

Posts (if posted with user and token) can be deleted by code@{ix-delete} command
which prompts for post id (the string after http://ix.io/)

curl is used as the backend via grapnel http request library.")
    (license #f)))

(define-public emacs-persp-projectile
  (package
    (name "emacs-persp-projectile")
    (version "20180616.1944")
    (source (origin
              (method url-fetch)
              (uri (string-append
                    "https://melpa.org/packages/persp-projectile-" version
                    ".el"))
              (sha256
               (base32
                "1pc1v6rc44wl7sq0qchgc4lxiy8lr7zvp0hygcp95c1x0dkqs8jl"))))
    (build-system emacs-build-system)
    (propagated-inputs `(("emacs-perspective" ,emacs-perspective)
                         ("emacs-projectile" ,emacs-projectile)))
    (home-page "https://github.com/bbatsov/persp-projectile")
    (synopsis "Perspective integration with Projectile")
    (description
     "This library bridges perspective mode to the awesome library
Projectile.  The idea is to create a separate perspective when
switching project.  A perspective is an independent workspace for
Emacs, similar to multiple desktops in Gnome and MacOS.  I often
work on many projects at the same time, and using perspective and
projectile together allows me to easily know which project I'm
current in, and focus on files that only belong to current project
when switching buffer.

To use this library, put this file in your Emacs load path, and
call (require 'persp-projectile)

See perspective.el on github: https://github.com/nex3/perspective-el
")
    (license license:gpl3+)))

(define-public emacs-helm-rg
  (let ((commit "ee0a3c09da0c843715344919400ab0a0190cc9dc")
        (revision "1"))
    (package
      (name "emacs-helm-rg")
      (version (git-version "0.1" revision commit))
      (source (origin
                (method git-fetch)
                (uri (git-reference
                      (url "https://github.com/cosmicexplorer/helm-rg")
                      (commit commit)))
                (file-name (git-file-name name version))
                (sha256
                 (base32
                  "0m4l894345n0zkbgl0ar4c93v8pyrhblk9zbrjrdr9cfz40bx2kd"))))
      (build-system emacs-build-system)
      (arguments
       `(#:phases (modify-phases %standard-phases
                    ;; TODO: Build is currently broken:
                    ;; https://github.com/cosmicexplorer/helm-rg/issues/20
                    (delete 'build))))
      (native-inputs `(("emacs" ,emacs-minimal)))
      (propagated-inputs `(("emacs-dash" ,emacs-dash)
                           ("emacs-helm" ,emacs-helm)))
      (home-page "https://github.com/cosmicexplorer/helm-rg")
      (synopsis "Helm interface to ripgrep")
      (description "Helm interface for ripgrep.")
      (license license:gpl3+))))

(define-public emacs-ace-link-2021-01-1
  (let ((commit "298f02f7dd117f9ec01f6aa2a2ddfecae0efb7f4")
        (revision "1"))
    (package
      (inherit emacs-ace-link)
      (name "emacs-ace-link-2021-01-1")
      (version (git-version "0.5.0" revision commit))
      (source (origin
                (method git-fetch)
                (uri (git-reference
                      (url "https://github.com/abo-abo/ace-link")
                      (commit commit)))
                (file-name (git-file-name name version))
                (sha256
                 (base32
                  "1i243wfwrbxn00sh96248lpqfb7cvxqqwlc78nf8kim4ymylpp41")))))))

(define-public emacs-ace-mc
  (let ((commit "6877880efd99e177e4e9116a364576def3da391b")
        (revision "1"))
    (package
      (name "emacs-ace-mc")
      (version (git-version "0.0.0" revision commit))
      (source (origin
                (method git-fetch)
                (uri (git-reference
                      (url "https://github.com/mm--/ace-mc")
                      (commit commit)))
                (file-name (git-file-name name version))
                (sha256
                 (base32
                  "180licc7w5b6f42ifwvllbnmf3aq8cbr8jhkbk37lzick4sv10d2"))))
      (build-system emacs-build-system)
      (propagated-inputs `(("emacs-dash" ,emacs-dash)
                           ("emacs-multiple-cursors" ,emacs-multiple-cursors)
                           ("emacs-ace-jump-mode" ,emacs-ace-jump-mode)))
      (home-page "https://github.com/mm--/ace-mc")
      (synopsis "Easily add and remove multiple cursors using ace jump mode")
      (description
       "Easily add and remove multiple cursors using ace jump mode.")
      (license license:gpl3+))))

(define-public emacs-org-roam-ui
  (let ((commit "5ac74960231db0bf7783c2ba7a19a60f582e91ab")
        (revision "4"))
    (package
      (name "emacs-org-roam-ui")
      (version (git-version "0.0.0" revision commit))
      (source (origin
                (method git-fetch)
                (uri (git-reference
                      (url "https://github.com/org-roam/org-roam-ui")
                      (commit commit)))
                (file-name (git-file-name name version))
                (sha256
                 (base32
                  "0yic5rgp4f1rmi979if79kva7wn3rqnky423mqgf7sdw310h8akl"))))
      (build-system emacs-build-system)
      (inputs `(("emacs" ,emacs)))
      (propagated-inputs `(("emacs-dash" ,emacs-dash)
                           ("emacs-simple-httpd" ,emacs-simple-httpd)
                           ("emacs-f" ,emacs-f)
                           ("emacs-websocket" ,emacs-websocket)
                           ("emacs-org" ,emacs-org)
                           ("emacs-org-roam" ,emacs-org-roam)))
      (arguments
       `(#:include (cons "out/" %default-include)
         ;; TODO: this really should be done as part of either the emacs-org package, or as part of another separate package
         #:phases (modify-phases %standard-phases
                    (add-after 'build 'install-application
                      (lambda* (#:key inputs outputs #:allow-other-keys)
                        (let* ((out (assoc-ref outputs "out"))
                               (emacs (assoc-ref inputs "emacs"))
                               (xapplications (string-append out
                                               "/share/applications")))
                          (mkdir-p xapplications)
                          (make-desktop-entry-file (string-append
                                                    xapplications
                                                    "/org-protocol.desktop")
                           #:name "Org-Protocol"
                           #:exec (format #f "~a/bin/emacsclient %u" emacs)
                           #:icon "emacs-icon"
                           #:comment
                           "Protocol which allows interaction between org-roam-server in emacs and a web browser"
                           #:categories '("Application")
                           #:mime-type "x-scheme-handler/org-protocol") #t))))))
      (home-page "https://github.com/org-roam/org-roam-ui")
      (synopsis
       "A graphical frontend for exploring your org-roam Zettelkasten")
      (description
       "A graphical frontend for exploring your org-roam Zettelkasten.")
      (license license:gpl3+))))

(define-public emacs-mu4e-dashboard
  (let ((commit "7956bf9631abfecdf4fbff168aa92064dc4886fb")
        (revision "1"))
    (package
      (name "emacs-mu4e-dashboard")
      (version (git-version "0.0.0" revision commit))
      (source (origin
                (method git-fetch)
                (uri (git-reference
                      (url "https://github.com/rougier/mu4e-dashboard")
                      (commit commit)))
                (file-name (git-file-name name version))
                (sha256
                 (base32
                  "0fkbkdc269l4sbpvbnmqq02vl80dz80l3d2zq6ax6dwhlvxk2blp"))))
      (build-system emacs-build-system)
      (propagated-inputs `(("mu" ,mu)))
      (home-page "https://github.com/rougier/mu4e-dashboard")
      (synopsis
       "Provides mu4e org-mode link type for building org powered dashboards")
      (description
       "Provides a new mu4e org link type that allows for mu4e queries from
org-mode.  These links can then be organized to form a org-mode powered mu4e
dashboard!")
      (license license:gpl3+))))

(define-public emacs-org-timeline
  (package
    (name "emacs-org-timeline")
    (version "0.4.0")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/Fuco1/org-timeline")
                    (commit version)))
              (file-name (git-file-name name version))
              (sha256
               (base32
                "106603835m3dy3bzjiasq2w574faxnn5kb72gr0y0mdkd0iwh8qa"))))
    (build-system emacs-build-system)
    (propagated-inputs `(("emacs-dash" ,emacs-dash)
                         ("emacs-org" ,emacs-org)))
    (home-page "https://github.com/Fuco1/org-timeline")
    (synopsis "Add graphical view of agenda to agenda buffer")
    (description "Add graphical view of agenda to agenda buffer.")
    (license license:gpl3+)))

(define-public emacs-org-html-themes
  (let ((commit "20f84182239a537f4c6b533eb65daa2d342428dd")
        (revision "1"))
    (package
      (name "emacs-org-html-themes")
      (version (git-version "1.2.0" revision commit))
      (source (origin
                (method git-fetch)
                (uri (git-reference
                      (url "https://github.com/fniessen/org-html-themes")
                      (commit commit)))
                (file-name (git-file-name name version))
                (sha256
                 (base32
                  "1jzkfsq4i4w2cgnlaw8cxa0b33rf8gy82bkbvl52hzlzhp68krgv"))))
      (build-system copy-build-system)
      (arguments
       `(#:phases (modify-phases %standard-phases
                    (add-after 'install 'hardcode-setup-file-paths
                      (lambda* (#:key outputs #:allow-other-keys)
                        (let* ((out (assoc-ref outputs "out"))
                               (src (string-append out
                                     "/var/org-themes/org-html-themes/src/")))
                          (for-each (lambda (setup-file)
                                      (substitute* setup-file
                                        (("src=\"src/")
                                         (string-append "src=\"" src))
                                        (("href=\"src/")
                                         (string-append "href=\"" src))))
                                    (find-files out "\\.setup$")) #t))))
         #:install-plan
         ;; TODO: src contains a bunch of minified javascript, which blocks
         ;; this package from being merged upstream until resolved
         '(("src" "var/org-themes/org-html-themes/src")
           ("org/" "var/org-themes/org-html-themes"
            #:include-regexp ("-local\\.setup$")))))
      (home-page "https://github.com/fniessen/org-html-themes")
      (synopsis "Set of org html export themes")
      (description "Export Org mode files into awesome HTML in 2 minutes.")
      (license license:gpl3+))))

(define-public emacs-org-html-theme-darksun
  (package
    (name "emacs-org-html-theme-darksun")
    (version "0.1")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/alphapapa/org-html-theme-darksun")
                    (commit version)))
              (file-name (git-file-name name version))
              (sha256
               (base32
                "0vncqab5ad6cbgqfq4qrf070q11858mvi0zhmji5ij2a233m3an6"))))
    (build-system copy-build-system)
    (arguments
     `(#:phases (modify-phases %standard-phases
                  (add-after 'install 'hardcode-setup-file-paths
                    (lambda* (#:key outputs #:allow-other-keys)
                      (let* ((out (assoc-ref outputs "out"))
                             (src (string-append out
                                   "/var/org-themes/org-html-theme-darksun/export/")))
                        (for-each (lambda (setup-file)
                                    (substitute* setup-file
                                      (("src=\"export/")
                                       (string-append "src=\"" src))
                                      (("href=\"export/")
                                       (string-append "href=\"" src))))
                                  (find-files out "\\.setup$")) #t))))
       #:install-plan
       ;; TODO: src contains a bunch of minified javascript, which blocks
       ;; this package from being merged upstream until resolved
       '(("export/styles"
          "var/org-themes/org-html-theme-darksun/export/styles")
         ("export/setup/" "var/org-themes/org-html-theme-darksun"
          #:include-regexp ("-local\\.setup$")))))
    (home-page "https://github.com/alphapapa/org-html-theme-darksun")
    (synopsis "Solarized Dark version of the Bigblow Org HTML export theme")
    (description
     "Solarized Dark version of the Bigblow Org HTML export theme.")
    (license license:gpl3+)))

;; TODO: send upsteam
(define-public emacs-eval-in-repl
  (package
    (name "emacs-eval-in-repl")
    (version "20201121.1341")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/kaz-yos/eval-in-repl")
                    (commit "2abb9ccf6f08ae3a5ab504f0b3fd81ce0345b766")))
              (sha256
               (base32
                "1707c7v6n2w94nnlb6n08y2m11bh7abm6fks2zrf0i80srxb8mrk"))))
    (build-system emacs-build-system)
    (propagated-inputs `(("emacs-dash" ,emacs-dash)
                         ("emacs-paredit" ,emacs-paredit)
                         ("emacs-ace-window" ,emacs-ace-window)
                         ("emacs-cider" ,emacs-cider)
                         ("emacs-elm-mode" ,emacs-elm-mode)
                         ("emacs-erlang" ,emacs-erlang)
                         ("emacs-geiser" ,emacs-geiser)
                         ("emacs-hy-mode" ,emacs-hy-mode)
                         ("emacs-elixir-mode" ,emacs-elixir-mode)
                         ("emacs-alchemist" ,emacs-alchemist)
                         ("emacs-js-comint" ,emacs-js-comint)
                         ("emacs-lua-mode" ,emacs-lua-mode)
                         ("emacs-tuareg" ,emacs-tuareg)
                         ("emacs-racket-mode" ,emacs-racket-mode)
                         ("emacs-inf-ruby" ,emacs-inf-ruby)
                         ("emacs-slime" ,emacs-slime)
                         ("emacs-sly" ,emacs-sly)
                         ("emacs-sml-mode" ,emacs-sml-mode)))
    (home-page "https://github.com/kaz-yos/eval-in-repl")
    (synopsis "Consistent ESS-like eval interface for various REPLs")
    (description
     "Provides a consistent ESS-like eval interface for various REPLs

This package does what ESS does for R for various REPLs, including ielm.

Emacs Speaks Statistics (ESS) package has a nice function called
ess-eval-region-or-line-and-step, which is assigned to C-RET.
This function sends a line or a selected region to the corresponding
shell (R, Julia, Stata, etc) visibly. It also start up a shell if
there is none.

This package along with REPL/shell specific packages implement similar
work flow for various REPLs.")
    (license license:expat)))

(define-public emacs-alchemist
  (package
    (name "emacs-alchemist")
    (version "20180312.1304")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/tonini/alchemist.el")
                    (commit "6f99367511ae209f8fe2c990779764bbb4ccb6ed")))
              (sha256
               (base32
                "12f95rwxs11sqf1w9pnf6cxc2lh2jz4nqkq33p8b5yamnl8cq9kg"))))
    (build-system emacs-build-system)
    (propagated-inputs `(("emacs-elixir-mode" ,emacs-elixir-mode)
                         ("emacs-dash" ,emacs-dash)
                         ("emacs-company" ,emacs-company)
                         ("emacs-pkg-info" ,emacs-pkg-info)
                         ("emacs-s" ,emacs-s)))
    (arguments
     '(#:include '("^[^/]+.el$" "^[^/]+.el.in$"
                   "^dir$"
                   "^[^/]+.info$"
                   "^[^/]+.texi$"
                   "^[^/]+.texinfo$"
                   "^doc/dir$"
                   "^doc/[^/]+.info$"
                   "^doc/[^/]+.texi$"
                   "^doc/[^/]+.texinfo$"
                   "^[^/]+.exs$"
                   "^alchemist-server$")
       #:exclude '("^.dir-locals.el$" "^test.el$" "^tests.el$"
                   "^[^/]+-test.el$" "^[^/]+-tests.el$")))
    (home-page "http://www.github.com/tonini/alchemist.el")
    (synopsis "Elixir tooling integration into Emacs")
    (description
     "Alchemist brings you all the Elixir tooling and power inside your Emacs editor.

Alchemist comes with a bunch of features, which are:

@itemize
@item Mix integration
@item Compile & Execution of Elixir code
@item Inline code evaluation
@item Inline macro expanding
@item Documentation lookup
@item Definition lookup
@item Powerful IEx integration
@item Smart code completion
@item Elixir project management
@item Phoenix support
@end itemize")
    (license license:gpl3)))

;; TODO: send upstream
(define-public emacs-rcirc-notify
  (let ((commit "841a7b5a6cdb0c11a812df924d2c6a7d364fd455")
        (revision "1"))
    (package
      (name "emacs-rcirc-notify")
      (version (git-version "20150219.2204" revision commit))
      (source (origin
                (method git-fetch)
                (uri (git-reference
                      (url "https://github.com/nicferrier/rcirc-notify")
                      (commit commit)))
                (sha256
                 (base32
                  "1k4knsrca626pikgaalqbqwy7im4wz1vrmzzhdrdb4lhdz6sq3q3"))))
      (build-system emacs-build-system)
      (home-page "https://github.com/nicferrier/rcirc-notify")
      (synopsis "Provide libnotify popup notifications for rcirc")
      (description
       "This extension is inspired in part by @code{erc-page-me.el} and offers
the same functionality as it, but for rcirc.")
      (license license:gpl2))))

(define-public emacs-flycheck-hledger
  (package
    (name "emacs-flycheck-hledger")
    (version "0.3.0")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url
                     "https://github.com/DamienCassou/flycheck-hledger.git")
                    (commit (string-append "v" version))))
              (sha256
               (base32
                "0q6b0vyd55x71kv4k7jp8xzgy5zf9md8mgy3y6pwmpga9dmv149j"))))
    (build-system emacs-build-system)
    (propagated-inputs `(("emacs-flycheck" ,emacs-flycheck)))
    (home-page "https://github.com/DamienCassou/flycheck-hledger/")
    (synopsis "Flycheck module to check hledger journals")
    (description "This package is a flycheck checker for hledger files.")
    (license license:gpl3)))

(define-public emacs-lsp-haskell
  (package
    (name "emacs-lsp-haskell")
    (version "20220307.2312")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/emacs-lsp/lsp-haskell.git")
                    (commit "daa51072e1718ca075987901fccbbc2357bca1fc")))
              (sha256
               (base32
                "0d2myk2906j0ngivca1yf4vdi8bfk5pz706nx279bf0pil7irdy0"))))
    (build-system emacs-build-system)
    (propagated-inputs (list emacs-lsp-mode))
    (home-page "https://github.com/emacs-lsp/lsp-haskell")
    (synopsis "Haskell support for lsp-mode")
    (description "Haskell specific adapter for LSP mode")
    (license license:gpl3)))

(define-public emacs-org-category-capture
  (package
    (name "emacs-org-category-capture")
    (version "20220114.730")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/IvanMalison/org-projectile.git")
                    (commit "642b39c698db00bc535c1c2335f425fb9f4855a9")))
              (sha256
               (base32
                "1pcfyrmbnsk39w4d6cs27bcyihics3gll515fslnl5kqbaj9qn09"))))
    (build-system emacs-build-system)
    (propagated-inputs (list emacs-org))
    (arguments
     '(#:include '("^org-category-capture.el$")
       #:exclude '()))
    (home-page "https://github.com/IvanMalison/org-projectile")
    (synopsis "Contextualy capture of org-mode TODOs.")
    (description
     "This package provides an interface that can be used to capture TODOs with a
category that is selected depending on a some piece of Emacs context.")
    (license license:gpl3)))

(define-public emacs-org-projectile
  (package
    (name "emacs-org-projectile")
    (version "20220114.730")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/IvanMalison/org-projectile.git")
                    (commit "642b39c698db00bc535c1c2335f425fb9f4855a9")))
              (sha256
               (base32
                "1pcfyrmbnsk39w4d6cs27bcyihics3gll515fslnl5kqbaj9qn09"))))
    (build-system emacs-build-system)
    (propagated-inputs (list emacs-projectile emacs-dash emacs-s
                             emacs-org-category-capture))
    (arguments
     '(#:include '("^org-projectile.el$")
       #:exclude '()))
    (home-page "https://github.com/IvanMalison/org-projectile")
    (synopsis "Repository todo management for org-mode")
    (description
     "This package aims to provide an easy interface to creating per project org-mode
TODO headings.")
    (license license:gpl3)))

(define-public emacs-org-projectile-helm
  (package
    (name "emacs-org-projectile-helm")
    (version "20180601.1822")
    (source (origin
              (method git-fetch)
              (uri (git-reference
                    (url "https://github.com/IvanMalison/org-projectile.git")
                    (commit "642b39c698db00bc535c1c2335f425fb9f4855a9")))
              (sha256
               (base32
                "1pcfyrmbnsk39w4d6cs27bcyihics3gll515fslnl5kqbaj9qn09"))))
    (build-system emacs-build-system)
    (propagated-inputs (list emacs-org-projectile emacs-helm emacs-helm-org))
    (arguments
     '(#:include '("^org-projectile-helm.el$")
       #:exclude '()))
    (home-page "https://github.com/IvanMalison/org-projectile")
    (synopsis "helm functions for org-projectile")
    (description
     "This package aims to provide an easy interface to creating per project org-mode
TODO headings.")
    (license license:gpl3)))
